/*
 * Wartezimmer.java - Beispiel fr den Einsatz von PriorityQueue mit Comparable 
 *
 */

import java.util.*;
import java.text.DateFormat;

class Patient implements Comparable<Patient> {
   private String name;
   private Date termin;
   private boolean notfall;

   Patient(String name, Date termin, boolean notfall) {
      this.name = name;
      this.termin = termin;
      this.notfall = notfall;
   }

   public int compareTo(Patient p) {
      if (notfall)
         return -1;
      else
         return this.termin.compareTo(p.termin);
   }

   public String toString() {
      if (notfall)
         return "** Notfall! ** \t " + this.name;
      else {
         DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
         DateFormat tf = DateFormat.getTimeInstance(DateFormat.SHORT);

         return df.format(this.termin) + " " + tf.format(this.termin) 
                + " \t " + this.name;
      }
   }
}


public class Wartezimmer {

   public static void main(String[] args) {
      System.console().printf("\n"); 

      PriorityQueue<Patient> patienten = new PriorityQueue<Patient>();

      Calendar c = Calendar.getInstance(Locale.GERMANY);
      c.set(2004, Calendar.OCTOBER, 12, 9, 15);
      patienten.offer( new Patient("Peter Mller", c.getTime(), false));

      c.set(2004, Calendar.OCTOBER, 12, 9, 30);
      patienten.offer( new Patient("Dirk Louis", c.getTime(), false));

      patienten.offer( new Patient("anonym", new Date(), true));

      c.set(2004, Calendar.OCTOBER, 12, 8, 30);
      patienten.offer( new Patient("Mark Tech", c.getTime(), false));

      for (Patient p : patienten) {
         System.console().printf(" %s\n", p); 
      }
   }
}
