/*
 * FileCopyVergleich.java - Kopiert eine Datei mit NIO
 *
 */

import java.io.*;
import java.nio.*;
import java.nio.channels.*;

public class DateiKopieVergleich  {
   public static void main(String[] args)  {
      System.out.println(); 

      if(args.length != 2)  {
         System.out.println(" Falsche Anzahl Argumente in Befehlszeile");
         System.out.println(" Aufruf mit <Quelldatei> <Zieldatei>");
         System.exit(0); 
      }

      variante_1(args[0], args[1]);  
      variante_2(args[0], args[1], false);  
      variante_2(args[0], args[1], true);  
      variante_3(args[0], args[1]);  
      variante_4(args[0], args[1]);  
   }

   // Kopieren mit BufferedInputStream/OutputStream 
   static void variante_1(String quelle, String ziel)  {
      try {
         long start = System.currentTimeMillis(); 

         // Eingabedatei ffnen
         BufferedInputStream eingabe = new BufferedInputStream(new FileInputStream(quelle));

         // Ausgabedatei ffnen
         BufferedOutputStream ausgabe = new BufferedOutputStream(new FileOutputStream(ziel)); 

         int b;

         while( (b = eingabe.read()) != -1)  {
            ausgabe.write(b); 
         }

         eingabe.close();
         ausgabe.close();

         long ende = System.currentTimeMillis(); 
         System.out.println(" BufferedInputStream/OutputStream: " + (ende - start) + " ms."); 
      } catch(Exception e) {
         System.err.println(e); 
      }
   }

   // Kopieren mit FileChannel und ByteBuffer
   static void variante_2(String quelle, String ziel, boolean direkt)  {
      try  {
         long start = System.currentTimeMillis(); 

         // Eingabedatei ffnen
         FileInputStream eingabeDatei = new FileInputStream(quelle); 
         FileChannel eingabe = eingabeDatei.getChannel();

         // Zieldatei ffnen
         FileOutputStream ausgabeDatei = new FileOutputStream(ziel);
         FileChannel ausgabe = ausgabeDatei.getChannel(); 

         ByteBuffer puffer;

         if(direkt == false)
            puffer = ByteBuffer.allocate(2048); 
         else
            puffer = ByteBuffer.allocateDirect(2048); 

         int num;


         while( (num = eingabe.read(puffer)) != -1)  {
            puffer.flip(); 
            ausgabe.write(puffer); 
            puffer.compact(); 
         }


         eingabeDatei.close();
         ausgabeDatei.close(); 

         long ende = System.currentTimeMillis(); 

         if(direkt == false)
            System.out.println(" FileChannel/ByteBuffer         : " + (ende - start) + " ms."); 
         else
            System.out.println(" FileChannel/direkter ByteBuffer: " + (ende - start) + " ms."); 
      } catch(Exception e) {
         System.err.println(e.getMessage()); 
      }
   }

   // Kopieren mit FileChannel und transferTo
   static void variante_3(String quelle, String ziel)  {
      try  {
         long start = System.currentTimeMillis(); 

         // Eingabedatei ffnen
         FileInputStream eingabeDatei = new FileInputStream(quelle); 
         FileChannel eingabe = eingabeDatei.getChannel();

         // Zieldatei ffnen
         FileOutputStream ausgabeDatei = new FileOutputStream(ziel);
         FileChannel ausgabe = ausgabeDatei.getChannel(); 


         // die Lnge der zu kopierenden Datei
         long num = eingabe.size(); 
         long startPos = eingabe.position(); // sollte 0 sein 

         // kopieren 
         eingabe.transferTo(startPos,num,ausgabe); 

         eingabe.close();
         ausgabe.close(); 

         long ende = System.currentTimeMillis(); 
         System.out.println(" FileChannel + transferTo(): " + (ende - start) + " ms."); 
      } catch(Exception e) {
         System.err.println(e.getMessage()); 
      }
   }

   static void variante_4(String quelle, String ziel)  {
      try  {
         long start = System.currentTimeMillis(); 

         // Eingabedatei ffnen
         FileInputStream eingabeDatei = new FileInputStream(quelle); 
         FileChannel eingabe = eingabeDatei.getChannel();

         // Zieldatei ffnen
         RandomAccessFile ausgabeDatei = new RandomAccessFile(ziel,"rw");
         FileChannel ausgabe = ausgabeDatei.getChannel(); 

         // Memory Mapping durchfhren 
         long groee = eingabe.size(); 
         MappedByteBuffer pufferEin = eingabe.map(FileChannel.MapMode.READ_ONLY, 0, groee);
         MappedByteBuffer pufferAus = ausgabe.map(FileChannel.MapMode.READ_WRITE, 0, groee);

         // FileChannel knnen bereits geschlossen sein 
         ausgabeDatei.close();
         eingabeDatei.close();  

         // kopieren
         while(pufferEin.hasRemaining())  {
            pufferAus.put(pufferEin.get()); 
         }

         long ende = System.currentTimeMillis(); 
         System.out.println(" Memory Mapping: " + (ende - start) + " ms."); 
      } catch(Exception e) {
         System.err.println(e.getMessage()); 
      }
   }
}
