/**
 * VerzeichnisKopierer.java - Kopiert Verzeichnisse inkl. Inhalt in ein Zielverzeichnis hinein
 */
import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;

public class VerzeichnisKopierer extends SimpleFileVisitor {
   private Path zielVerzeichnis;
   private Path quelleVater;
   
   public void kopiere(Path quelle, Path ziel) throws IOException {
      if(Files.isDirectory(quelle) && Files.isDirectory(ziel) 
                                   && !Files.isSameFile(quelle, ziel)) {
         quelleVater = quelle.toAbsolutePath().getParent();
         zielVerzeichnis  = ziel.toAbsolutePath();
         Files.walkFileTree(quelle, this );
      }
      else {
         throw new IOException("Ungltige Quelle/Ziel");
      }
   }

   private Path erzeugeZielPfad(Path quellPfad) {
      Path quelleRelativ    = quelleVater.relativize(quellPfad);
      return zielVerzeichnis.resolve(quelleRelativ); 
   }
   
   public FileVisitResult preVisitDirectory(Object t, 
	                                        BasicFileAttributes bfa) 
											         throws IOException {
      Path unterverzeichnis = erzeugeZielPfad((Path) t);

      // Zielverzeichnis anlegen
      Files.createDirectories(unterverzeichnis);

      return FileVisitResult.CONTINUE;
   }

   public FileVisitResult visitFile(Object t, 
	                                 BasicFileAttributes bfa) 
                                    throws IOException {
      Path quelle           = (Path) t;
      Path zielDatei        = erzeugeZielPfad(quelle); 

      Files.copy((Path) t, zielDatei);
      return FileVisitResult.CONTINUE;
   }
   
   public static void main(String ... args) {
      if(args.length != 2) {
            System.out.println("Parameter: Quellverzeichnis Zielverzeichnis");
      }
      else {
         VerzeichnisKopierer vk = new VerzeichnisKopierer();
         Path quelle = Paths.get(args[0]);
         Path ziel   = Paths.get(args[1]);

         try {
            vk.kopiere(quelle, ziel);
         }
         catch(IOException ex) {
            System.err.println(ex.getMessage());
         }
      }
   }
}
