/* 
 * JTableDemo3.java - Beispiel fr JTable mit eigenem TableModel
 *                    sowie Editier/Selektionsmglichkeit auf Zellen
 */

import javax.swing.*;
import javax.swing.event.*; 
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;

public class JTableDemo3 extends JFrame 
                         implements ListSelectionListener, TableModelListener { 

   // Instanzvariablen, damit in den Listener-Methoden darauf zugegriffen werden kann
   JTable tabelle; 
       
   public JTableDemo3() {
      super();

      // Hauptfenster einrichten
      setTitle("JTable Demo3"); 
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	  
      // Tabelle anlegen mit Einzelselektion von Zeilen 
      TabellenModell modell = new TabellenModell();
      tabelle = new JTable(modell);
      tabelle.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); // nur einzelne Zeilen
      tabelle.setColumnSelectionAllowed(false); 
      tabelle.setRowSelectionAllowed(true); 
      getContentPane().add(new JScrollPane(tabelle)); 

      TableColumnModel spaltenModell = tabelle.getColumnModel();

      // Ereignisbehandlung registrieren
      // 1. fr nderung der selektierten Zeile/Spalte
      ListSelectionModel smZeile = tabelle.getSelectionModel(); 
      smZeile.addListSelectionListener(this);

      // 2. fr Mausklicks rechte Maustaste
      tabelle.addMouseListener(new MausAdapter(tabelle));  

      // 3. fr nderung des Tabelleninhalts
      tabelle.getModel().addTableModelListener(this);  


   }

   // Hier Ereignisbehandlungsmethoden fr Komponenten
   // Selektionsnderung 
   public void valueChanged(ListSelectionEvent e) {

       if(e.getValueIsAdjusting() == true)
	  return;

       System.out.println("Aktuell selektierte Zeile: " +  tabelle.getSelectedRow());
   } 

   // Tabelleninhalt nderung
   public void tableChanged(TableModelEvent e) {
      System.out.println(e); 
	   
   }




   public static void main(String args[]) {
      // Fenster erzeugen und anzeigen
      JTableDemo3 hauptfenster = new JTableDemo3();
      hauptfenster.setSize(400,200);                      
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true); 
   }
    
}


// eigenes Tabellenmodell fr die Daten
class TabellenModell extends AbstractTableModel {

    // Tabellendaten bereitstellen  
    Object[][] daten = {
              {"Erika","Mustermann", new Integer(24), new Boolean(false)},
              {"Dirk","Louis", new Integer(54), new Boolean(true)},
              {"Peter","Mller", new Integer(49), new Boolean(false)},
              {"Kurt","Meier", new Integer(17), new Boolean(false)},
              {"Anna","Trobo", new Integer(21), new Boolean(true)}}; 

    String[] spaltenNamen = {"Vorname", "Nachname", "Alter", "Verheiratet"}; 

    public int getRowCount() {
       return daten.length;
    }

    public int getColumnCount() {
       return spaltenNamen.length;
    }

    public String getColumnName(int num) {
       return spaltenNamen[num];
    }

    public void setValueAt(Object obj, int x, int y) {
       daten[x][y] = obj; 

       fireTableDataChanged(); // Listener benachrichtigen, dass nderung erfolgt ist
    }

    // alle Zeilen und Spalten editierbar  
    public boolean isCellEditable(int x, int y) {
       return true;
    }

    public Object getValueAt(int x, int y) {
       return daten[x][y]; 
    }

    // Class-Objekt wird fr den DefaultRenderer verwendet
    //
    public Class getColumnClass(int num) {
       Object obj = getValueAt(0,num);
       return obj.getClass();  
    }
    
}


// Mausadapter fr Mausereignisse
class MausAdapter extends MouseAdapter {
    private JTable tabelle;

    MausAdapter(JTable tab) {
	super();
	tabelle = tab; 
    }

    public void mouseClicked(MouseEvent e) {
       if(e.getButton() == MouseEvent.BUTTON3) {
	  // rechte Maustaste gedrckt
	  Point p = e.getPoint();

	  // in Tabellenkoordinaten umrechnen
	  int zeile = tabelle.rowAtPoint(p); 
	  int spalte = tabelle.columnAtPoint(p); 

	  System.out.println("Rechte Maus bei : Zeile " + zeile + " Spalte " + spalte);
       }
   }
}	
