/*
 * JTreeDateiemo.java - Beispiel fr JTree mit TreeSelectionListener
 * 
*/

import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*; 

public class JTreeDateiDemo extends JFrame  implements TreeSelectionListener {
   JTextArea infoArea;

   public JTreeDateiDemo(String wurzelVerzeichnis) {
      super();

      // Hauptfenster einrichten
      setTitle("JTree Datei-Demo"); 
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 
      // Baummodell aufbauen
      File start = new File(wurzelVerzeichnis); 
      DefaultMutableTreeNode wurzel = new DefaultMutableTreeNode(start.getName());
      erzeugeDateiBaum(start, wurzel); 

      // JTree erzeugen
      DefaultTreeModel modell = new DefaultTreeModel(wurzel);
      DefaultTreeSelectionModel tsm = new DefaultTreeSelectionModel();
      tsm.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);

      JTree baum = new JTree(modell); 
      baum.setSelectionModel(tsm);
      baum.setRootVisible(true);
      baum.addTreeSelectionListener(this); 


      // Anzeige der Komponenten
      JScrollPane baumPane = new JScrollPane(baum); 
      infoArea = new JTextArea();
      JScrollPane infoPane = new JScrollPane(infoArea); 
      JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                             baumPane, infoPane);
      splitPane.setDividerLocation(150); 

      Dimension minGroesse = new Dimension(300,300);        
      baumPane.setMinimumSize(minGroesse);
      infoPane.setMinimumSize(minGroesse);

      
      getContentPane().add(splitPane,BorderLayout.CENTER);

   }

   // Hier Ereignisbehandlungsmethoden fr Komponenten
   public void valueChanged(TreeSelectionEvent e) {
      TreePath aktuell = e.getNewLeadSelectionPath();

      if(aktuell != null) {
	 infoArea.setText("Pfad: " + aktuell.toString()); 
	 DefaultMutableTreeNode knoten = (DefaultMutableTreeNode) aktuell.getLastPathComponent();

	 if(knoten.isLeaf() == true) 
	    infoArea.append("\nSelektierter Knoten ist eine Datei.");
	 else
	    infoArea.append("\nSelektierter Knoten ist ein Verzeichnis.");

      }

   } 




   // Erzeugt rekursiv einen Baum zur Reprsentation einer Dateihierarchie
   private void erzeugeDateiBaum(File aktuell, DefaultMutableTreeNode knoten) {
      // alle Verzeichnisse  und Dateien als Kinder hinzufgen 
      File[] dateien = aktuell.listFiles();

      if(dateien == null) // eine Datei liegt vor 
         return;

      for(int i = 0; i < dateien.length; i++) {
        DefaultMutableTreeNode kind = new DefaultMutableTreeNode(dateien[i].getName());   
        knoten.add(kind);
        erzeugeDateiBaum(dateien[i], kind); 
        }
   }

   public static void main(String args[]) {
      // Fenster erzeugen und anzeigen
      if(args.length != 1) {
	 System.out.println("Aufruf: <Wurzelverzeichnis>");
	 System.exit(0); 
      }

      JTreeDateiDemo hauptfenster = new JTreeDateiDemo(args[0]);
      hauptfenster.setSize(400,300);                      
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true);
   }
    
}

