/*
 * TabSplitDemo.java - Beispiel zu JSplitPane und JTabbedPane 
 *          
*/

import java.awt.*; 
import java.awt.event.*; 
import javax.swing.*;
import javax.swing.event.*; 
import java.util.*;

public class TabSplitDemo extends JFrame  implements ListSelectionListener, ActionListener {
                       
   JLabel aktBild; 
   JList<String> dateiListe; 
   JTabbedPane tabbedPane;
   final JPopupMenu popupMenu = new JPopupMenu();
   HashMap<JButton, Component> schliessenButtons = new HashMap<JButton, Component>();
       
   public TabSplitDemo() {
      super();

      // Hauptfenster einrichten
      setTitle("Split-Tabbed-Pane Demo"); 

      // Register einrichten
      tabbedPane = new JTabbedPane();
      JSplitPane bildPanel = erzeugeBildPanel();
      JPanel datumPanel = erzeugeDatumPanel(); 
      tabbedPane.addTab("Datum", datumPanel);
      tabbedPane.addTab("Bilder", new ImageIcon("Zoom16.gif"), (Component) bildPanel); 


	  
      // Anzeige der Komponenten
      getContentPane().add(tabbedPane,BorderLayout.CENTER);
 
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }

   // erzeugt ein JPanel fr Datum                      
   JPanel erzeugeDatumPanel() {
     JPanel panel = new JPanel();
     JSpinner spin = new JSpinner(new SpinnerDateModel());
     panel.add(spin);

     String[] wochenTage = new String[] {"Montag", "Dienstag", "Mittwoch",
                                         "Donnerstag", "Freitag", "Samstag",
                                         "Sonntag"};
     spin = new JSpinner(new SpinnerListModel(wochenTage)); 
     panel.add(spin); 

     return panel; 
   }


   // erzeugt ein SplitPane  mit Datei-Liste und
   // Bildanzeige 
   JSplitPane erzeugeBildPanel() {
      // Liste mit Dateinamen 
      String[] dateiNamen = new String[] {"alice.gif", "caterpillar.gif",
                                          "rabbit.gif", "rabbit2.gif","hatter.gif","duke.gif"};
      dateiListe = new JList<String>(dateiNamen);
      dateiListe.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); 
      dateiListe.setSelectedIndex(0); 
      dateiListe.addListSelectionListener(this);
      JScrollPane dateiAnzeige = new JScrollPane(dateiListe); 

      // Bildanzeige 
      ImageIcon bildIcon = new ImageIcon((String) dateiListe.getSelectedValue());
      aktBild = new JLabel(bildIcon); 
      JScrollPane bildAnzeige = new JScrollPane(aktBild);
	  


      // Mindestgren sicherstellen
      Dimension minGroesse = new Dimension(150,100);        
      dateiAnzeige.setMinimumSize(minGroesse);
      bildAnzeige.setMinimumSize(minGroesse); 

      // SplitPane erzeugen 
      JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                             dateiAnzeige, bildAnzeige);
      splitPane.setDividerLocation(150); 
      splitPane.setOneTouchExpandable(true); 

	  
	  // Popup-Men fr  die Erzeugung von Registern mit aktuellen Bild
	  JMenuItem item = new JMenuItem("Bild als Register hinzufgen");
	  
	  item.addActionListener(new ActionListener() 
	     {
		   public void actionPerformed( ActionEvent e )
            {
            String name = (String) dateiListe.getSelectedValue(); 
			erzeugeRegister(name); 
            }
		 }
	  
	  );
	  
	  popupMenu.add(item);

	  
	  dateiListe.addMouseListener( new MouseAdapter() 
	     {
		 // Achtung: wenn auch auf anderen Plattformen als Windows, dann sollte eine analoge Methode mousePressed() implementiert werden!
	      public void mouseReleased( MouseEvent e ) {
            if(e.isPopupTrigger())
              popupMenu.show(e.getComponent(), e.getX(), e.getY());
	      }
		 }
	  );
	  
      return splitPane; 
   }

   
   // aktuelles Bild als neues Register hinzufgen 
   public void erzeugeRegister(String name) { 
      ImageIcon bild = new ImageIcon((String) dateiListe.getSelectedValue());
	  JPanel bildPanel = new JPanel();
	  bildPanel.add(new JLabel(bild));
	  
	  // dieses JPanel bildet die Registerlasche und besteht aus Bildnamen und Schlieen-Button
	  JPanel registerLasche = new JPanel();
	  registerLasche.setOpaque(false); // damit Hintergrund von der Lasche zu sehen ist
	  registerLasche.setLayout(new FlowLayout()); 
	  JLabel registerTitel = new JLabel(name);
	  ImageIcon xgif = new ImageIcon("xicon.gif");
	  Dimension dim = new Dimension(
                              xgif.getIconWidth() + 5,
                              xgif.getIconHeight() + 5);
	  JButton schliessenButton = new JButton(xgif);
	  schliessenButton.setPreferredSize(dim);
	  registerLasche.add(registerTitel); 
      registerLasche.add(schliessenButton);

			
      // wenn Register-Button gedrckt wird, Register entfernen			
	  schliessenButton.addActionListener(this);
	  
	  tabbedPane.addTab(null, bildPanel); // null als Registername, da wir unsere Komponente registerLasche nehmen
	  tabbedPane.setTabComponentAt(tabbedPane.getTabCount()-1, registerLasche);
	  schliessenButtons.put(schliessenButton, registerLasche); 
   }
   
   
 
  // Hier Ereignisbehandlungsmethode fr Schlieen einer dynamisch erzeugten Registers
    public void actionPerformed(ActionEvent e) {
	   Component c = schliessenButtons.get((JButton) e.getSource());
	   
	   if(c != null) {
	      schliessenButtons.remove((JButton) e.getSource());
		  int index = tabbedPane.indexOfTabComponent(c);
		  tabbedPane.removeTabAt(index); 
	   }
	}
 
   
  
   public void valueChanged(ListSelectionEvent e) {

      if (e.getValueIsAdjusting())
            return;
      
      // gewhltes Bild anzeigen
      ImageIcon bild = new ImageIcon((String) dateiListe.getSelectedValue());
      aktBild.setIcon(bild); 
      aktBild.revalidate(); 
   } 





   public static void main(String args[]) {
      // Fenster erzeugen und anzeigen
      TabSplitDemo hauptfenster = new TabSplitDemo();
      hauptfenster.setSize(400,300);                    
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true); 
   }
}

