/* 
 * MenuDemo.java - Beispiel fr Menleiste
 *
 */

import java.awt.*; 
import java.awt.event.*;
import javax.swing.*; 

public class MenuDemo extends JFrame 
	              implements ActionListener, ItemListener {

   // Hier Variablen fr Komponenten deklarieren
   // auf die im Programm laufend zugegriffen werden muss
   private JCheckBoxMenuItem menuDateiCISymbolleiste;  
   private ButtonGroup menuDateiModus; 
   private JToolBar symbolLeiste;
       
   public MenuDemo() {
      super();

      // Hauptfenster einrichten
      setTitle("Men-Demo");

      // Hier Komponenten einfgen
      
      // Menleiste  
      JMenuBar menuLeiste = erzeugeMenueLeiste();     
      setJMenuBar(menuLeiste);

      // Symbolleiste 
      symbolLeiste = erzeugeSymbolLeiste(); 
      getContentPane().add(symbolLeiste, BorderLayout.NORTH); 

      // Lauscher fr Mausereinisse im Fenster (-> Kontextmen)
      KontextMenuLauscher kml = new KontextMenuLauscher(this); 
      addMouseListener(kml); 
      
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);      
   }


   // Diese Methode stellt die Menleiste zusammen 
   private JMenuBar erzeugeMenueLeiste()  {
      JMenuBar menueLeiste = new JMenuBar();

      JMenu menuInfo = new JMenu("Programm-Info"); 


      // Men Datei
      JMenu menuDatei = new JMenu("Datei");
      JMenuItem menuDateiOeffnen = new JMenuItem("Datei ffnen", new ImageIcon("Open.gif"));
      menuDateiOeffnen.addActionListener(this); 
      JMenuItem menuDateiSpeichern = new JMenuItem("Datei speichern", new ImageIcon("Save.gif"));
      menuDateiSpeichern.addActionListener(this); 

      menuDateiModus = new ButtonGroup();
      JRadioButtonMenuItem  menuDateiRBNormal = new JRadioButtonMenuItem("Normal", true);
      JRadioButtonMenuItem  menuDateiRBProfi = new JRadioButtonMenuItem("Profi", false);
      JRadioButtonMenuItem  menuDateiRBGuru = new JRadioButtonMenuItem("Guru", false);
      menuDateiModus.add(menuDateiRBNormal);
      menuDateiModus.add(menuDateiRBProfi);
      menuDateiModus.add(menuDateiRBGuru);

      menuDateiCISymbolleiste = new JCheckBoxMenuItem("Symbolleiste"); 
      menuDateiCISymbolleiste.setSelected(true); 
      menuDateiCISymbolleiste.addItemListener(this); 

      // Untermen Datei/Extras
      JMenu menuDateiExtras = new JMenu("Extras"); 
      JMenuItem menuDateiExtrasPfade = new JMenuItem("Pfade",'P');
      menuDateiExtrasPfade.addActionListener(this); 
      JMenuItem menuDateiExtrasFarben = new JMenuItem("Farben",'F');
      menuDateiExtrasFarben.addActionListener(this); 
      JMenuItem menuDateiExtrasSprache = new JMenuItem("Sprache",'S');
      menuDateiExtrasSprache.addActionListener(this); 

      // die Accelerators fr die Eintrge Pfade, Farben, Sprache erzeugen
      KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_F1,0); 
      menuDateiExtrasPfade.setAccelerator(ks);
      ks = KeyStroke.getKeyStroke('F',Event.CTRL_MASK); 
      menuDateiExtrasFarben.setAccelerator(ks);
      ks = KeyStroke.getKeyStroke('S',Event.SHIFT_MASK); 
      menuDateiExtrasSprache.setAccelerator(ks);
      
      JMenuItem menuDateiBeenden = new JMenuItem("Programm beenden");
      menuDateiBeenden.addActionListener(this); 


      // das Men Datei zusammenstellen
      menuDatei.add(menuDateiOeffnen);
      menuDatei.add(menuDateiSpeichern);
      menuDatei.addSeparator();
      menuDatei.add(menuDateiRBNormal);
      menuDatei.add(menuDateiRBProfi);
      menuDatei.add(menuDateiRBGuru);
      menuDatei.addSeparator();
      menuDatei.add(menuDateiCISymbolleiste);
      menuDatei.add(menuDateiExtras);
      menuDatei.addSeparator();
      menuDatei.add(menuDateiBeenden);

      // das Untermen Extras zusammenstellen
      menuDateiExtras.add(menuDateiExtrasPfade); 
      menuDateiExtras.add(menuDateiExtrasFarben); 
      menuDateiExtras.add(menuDateiExtrasSprache); 

      // Menleiste zusammenstellen 
      menueLeiste.add(menuInfo);  
      menueLeiste.add(menuDatei);

      return menueLeiste; 
   }


   // Diese Methode erzeugt die Symbolleiste 
   private JToolBar erzeugeSymbolLeiste() {
      JToolBar symbolLeiste = new JToolBar();

      JButton btnOeffnen = new JButton(new ImageIcon("open.gif"));
      btnOeffnen.setActionCommand("Datei ffnen"); 
      btnOeffnen.setToolTipText("ffnet eine Datei"); 
      symbolLeiste.add(btnOeffnen);
      btnOeffnen.addActionListener(this); 

      JButton btnSpeichern = new JButton(new ImageIcon("save.gif"));
      btnSpeichern.setActionCommand("Datei speichern"); 
      btnSpeichern.setToolTipText("Speichert die aktuelle Datei"); 
      symbolLeiste.add(btnSpeichern);
      btnSpeichern.addActionListener(this); 
      return symbolLeiste;
   }


   // Hier Ereignisbehandlungsmethoden fr Komponenten

   public void actionPerformed(ActionEvent e) {
     if(e.getActionCommand().equals("Programm beenden"))
	 System.exit(0); 

     // fr die anderen nur das Kommando ausgeben
     System.out.println(e.getActionCommand());
   }

   public void itemStateChanged(ItemEvent e) {
      Object item = e.getItemSelectable();

      if(item == menuDateiCISymbolleiste) {
         if(e.getStateChange() == ItemEvent.DESELECTED) 
            getContentPane().remove(symbolLeiste);  
         else
            getContentPane().add(symbolLeiste,BorderLayout.NORTH);  

         repaint(); 
      }
   }
    
   // innere Klassen:

   // Kontextmen-Handling 
   protected final class KontextMenuLauscher extends MouseAdapter {
      MenuDemo fenster;

      KontextMenuLauscher(MenuDemo f) {
	  fenster = f; 
      }

      public void mouseReleased(MouseEvent e) {

         if(e.isPopupTrigger() == true) {
            // Kontextmen erzeugen  
	        erzeugeKontextMenue(e);  
	     }
      }

      public void mousePressed(MouseEvent e) {

         if(e.isPopupTrigger() == true) {
            // Kontextmen erzeugen  
	        erzeugeKontextMenue(e);  
	     }
      }



      // erzeugt ein Kontext-Men und zeigt es an
      private void erzeugeKontextMenue(MouseEvent e) {
        JPopupMenu popup = new JPopupMenu(); 

        JMenuItem popupKopieren = new JMenuItem("Kopieren");
        popupKopieren.addActionListener(fenster);

        JMenuItem popupEinfuegen = new JMenuItem("Einfgen");
        popupEinfuegen.addActionListener(fenster);

        JMenuItem popupEnde = new JMenuItem("Programm beenden");
        popupEnde.addActionListener(fenster);

        popup.add(popupKopieren);
        popup.add(popupEinfuegen);
	    popup.addSeparator();
        popup.add(popupEnde);

        // anzeigen an aktueller Mausposition
        popup.show(e.getComponent(), e.getX(), e.getY());
      }

   }



   public static void main(String args[]) {
      // Fenster erzeugen und anzeigen
      MenuDemo hauptfenster = new MenuDemo();
      hauptfenster.setSize(400,350);                  // oder pack()    
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true); 
   }
    
 }

