/* 
 * SpezielleGradienten.java 
 * 
 */

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class SpezielleGradienten extends JFrame implements MouseListener {

    private static final int BREITE = 400;
    private static final int HOEHE  = 300;
    private LinearGradientPanel linearPanel;
    private RadialGradientPanel radialPanel; 
    
    
    public SpezielleGradienten() {
        setTitle("Linear/Radial-Gradient Demo");
        setSize(BREITE, HOEHE);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLayout(new GridLayout(1,2));
        linearPanel = new LinearGradientPanel();
        
        radialPanel = new RadialGradientPanel(); 
        radialPanel.addMouseListener(this); 
        
        add(linearPanel); 
        add(radialPanel); 
    }


    public void mouseClicked(MouseEvent me) {
         if(me.getComponent() == radialPanel) {
            radialPanel.setzeMausPosition(me.getPoint());
            radialPanel.repaint();  
         }
    }

    public void mouseEntered(MouseEvent me) {
    }
    public void mouseExited(MouseEvent me) {
    }
    public void mousePressed(MouseEvent me) {
    }
    public void mouseReleased(MouseEvent me) {
    }

    private class RadialGradientPanel extends JPanel {
        private Point mausPosition;
        
        public RadialGradientPanel() {
            super();
            mausPosition =  new Point(BREITE / 4,HOEHE / 2);
        }
        private void setzeMausPosition(Point point) {
           mausPosition = point;                   
        }
        protected void paintComponent(Graphics g) {
          super.paintComponent(g);
          
           Graphics2D g2d = (Graphics2D) g.create();
           Color[] farben = new Color[]{Color.YELLOW, Color.ORANGE, Color.BLUE};
           float radius   = 0.5f *  HOEHE;
           float[] verteilung = new float[]{0.2f, 0.4f, 0.8f};
           RadialGradientPaint radialPaint = new RadialGradientPaint(mausPosition, radius, verteilung, farben);       
           g2d.setPaint(radialPaint);
           g2d.fillRect(0,0, getWidth(), getHeight());
           
           g2d.dispose();
        }
    }
    
    private class LinearGradientPanel extends JPanel {
        private Point mausPosition;
        
        public LinearGradientPanel() {
            super();
        }
        protected void paintComponent(Graphics g) {
           super.paintComponent(g);
                    
           Graphics2D g2d = (Graphics2D) g.create();
           Point start = new Point(0, 0); 
           Point ende  = new Point(50,50);
           float[] verteilung = {0.0f, 0.2f, 0.7f};
           Color[] farben = {Color.BLACK, Color.RED, Color.YELLOW};

           LinearGradientPaint linearPaint;
           linearPaint =  new LinearGradientPaint(start, ende, verteilung, 
                                                  farben, MultipleGradientPaint.CycleMethod.REPEAT);
           g2d.setPaint(linearPaint);
           g2d.fillRect(0,0, getWidth() , getHeight());

           g2d.dispose();
        }
    }
    
    public static void main(String[] args) {
        SpezielleGradienten demo = new SpezielleGradienten();
        demo.setVisible(true);
    }
}
