/* 
 * Liniengrafik.java - Zeichnen in Swing-Komponenten
 * 
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;

public class Liniengrafik extends javax.swing.JFrame {
   private Leinwand leinwand;
   private JPanel   schalterfeld;
   private JButton  btnPositionieren;
   private JButton  btnZeichnen;
   private boolean  wirdPositioniert = true;

   private final double GESCHW = 0.1;         // Geschwindigkeit, in der Spirale zugeht
   private final int    WINDUNGEN = 30;       // Anzahl der Sprialwindungen

   private Vector<Point> positionen = new Vector<Point>();


   // innere Klasse fr Zeichenflche
   private class Leinwand extends JPanel {

      public Leinwand() {
         addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
               if( !wirdPositioniert ) 
                  return;

               Point p = new Point(e.getX(), e.getY());
               positionen.add(p);

               Graphics g = getGraphics();
               g.fillOval(p.x-6, p.y-6, 12, 12);
               g.dispose();

               btnZeichnen.setEnabled(true);            
            }
         });
      }
      
      public void paintComponent(Graphics g) {
         super.paintComponent(g);

         if( wirdPositioniert ) {    // nur Positionen zeigen
            Point p;
            for(int i = 0; i < positionen.size(); ++i) {
               p = positionen.get(i);
               g.fillOval(p.x-6, p.y-6, 12, 12);
            }
         } else {
            // lokale Kopie anlegen, damit Grafik jederzeit rekonstruierbar
            Point[] punkte = new Point[positionen.size()];
            for(int i = 0; i < punkte.length; ++i) {
               punkte[i] = new Point( ( positionen.get(i)).x,
                                      ( positionen.get(i)).y);
            }

            Point start, ziel;

            // Spirale erzeugen
            for(int w = 0; w < WINDUNGEN; ++w) {
               for(int i = 0; i < punkte.length; ++i) {
                  start = punkte[i];
                  ziel  = punkte[(i+1) % punkte.length];
                  g.drawLine(start.x, start.y, ziel.x, ziel.y);
               }

               // neue Positionen fr nchste Windung
               Point startpunkt = punkte[0];          // Startpunkt sichern

               Point p, pnext;
               for(int i = 0; i < punkte.length-1; ++i) {
                  p = punkte[i];
                  pnext = punkte[i+1];
                  punkte[i].x  += (int) ((pnext.x - p.x) * GESCHW);
                  punkte[i].y  += (int) ((pnext.y - p.y) * GESCHW);
               }
               p = punkte[punkte.length-1];
               pnext = startpunkt;
               punkte[punkte.length-1].x  += (int) ((pnext.x - p.x) * GESCHW);
               punkte[punkte.length-1].y  += (int) ((pnext.y - p.y) * GESCHW);
            }   
         }

      }
   }

       
   public Liniengrafik() {
      // Hauptfenster einrichten
      setTitle("Liniengrafiken");
      getContentPane().setBackground(Color.LIGHT_GRAY);
     
      // JPanel fr Leinwand
      leinwand = new Leinwand();
      leinwand.setBorder(BorderFactory.createEtchedBorder());
      leinwand.setBackground(Color.WHITE);

      // Schalterfeld
      schalterfeld = new JPanel();
      btnPositionieren = new JButton("Positionieren");
      btnPositionieren.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            positionenFestlegen();
         }
      });
      btnZeichnen = new JButton("Zeichnen");
      btnZeichnen.setEnabled(false);
      btnZeichnen.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            zeichnen();
         }
      });
      schalterfeld.add(btnPositionieren);
      schalterfeld.add(btnZeichnen);

      getContentPane().add(leinwand, BorderLayout.CENTER);
      getContentPane().add(schalterfeld, BorderLayout.SOUTH);

      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }

   /*
    * Ereignisbehandlungsmethoden
    *
    */
 
   private void positionenFestlegen() {
      positionen = new Vector<Point>();
      wirdPositioniert = true;
      leinwand.repaint();
   }

   private void zeichnen() {
      wirdPositioniert = false;
      btnZeichnen.setEnabled(false);
      leinwand.repaint();
   }

   public static void main(String args[]) {
      Liniengrafik hauptfenster = new Liniengrafik();
      hauptfenster.setSize(400,300);                     
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true); 
   }
    
}
