/* 
 * FileTransferHandler.java - TransferHandler fr JTextArea-Komponenten, 
 *                            der Dateien untersttzt
 *
 */
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

class FileTransferHandler extends TransferHandler {

  FileTransferHandler() {
  }


  // Prft, ob die angebotenen Daten in einem DataFlavor 
  // angeboten werden, der fr die Target-Komponente geeignet ist.
  public boolean canImport(TransferHandler.TransferSupport support) {

    if (!(support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)))
       return false;
    else     
       return true;
  }


  // versucht die Daten aus dem Transferable-Objekt t in der Komponente
  // abzulegen
  public boolean importData(TransferHandler.TransferSupport support) {
    
    // Vorab prfen, ob die angebotenen Daten in 
    // einem Data Flavor angeboten werden, der fr die Target-Komponente
    // geeignet ist.
    // sollte wegen mglicher Paste-Operation erneut berprft werden
    if (!canImport(support)) 
       return false;

    if (!(support.getComponent() instanceof JTextArea))
       return false;
            
    JTextArea target = (JTextArea) support.getComponent();
    Transferable t = support.getTransferable();        
                
    try {
       // Transferable-Objekt enthlt eine (oder mehrere) Dateien.
       // Lese die erste Datei und fge ihren Inhalt in die 
       // Target-Komponente ein

       FileReader in = null;
       StringBuilder str = new StringBuilder();
       java.util.List files = (java.util.List) 
            t.getTransferData(DataFlavor.javaFileListFlavor);

       if (files.size() > 0) {
           // nur erste Datei lesen
           File f = (File) files.get(0);

           try {
              in = new FileReader(f);

              // Dateiinhalt in String lesen
              int countBytes = 0;
              char[] bytesRead = new char[512];
       
              while( (countBytes = in.read(bytesRead)) > 0) 
                  str.append(bytesRead, 0, countBytes);   

              // Drop-Position bestimmen
              JTextComponent.DropLocation dropPos = 
                (JTextComponent.DropLocation) support.getDropLocation();
              int pos = dropPos.getIndex();
              target.setText(str.toString());
                    
           } catch (IOException e) {
              System.console().printf(" Datei-Fehler\n");
           } finally {
              if (in != null) 
                  in.close();
           }
        }
        return true;
        
    } catch (UnsupportedFlavorException e) {
       System.console().printf(" Drag-Daten werden nicht untersttzt\n");
    } catch (IOException e) {
       System.console().printf(" I/O-Exception\n");
    }
    return false;
  }
}
