/*
 * ClipDemo.java - Abspielen einer Sound-Datei via Clip
 *
 */
import java.io.*;           
import javax.sound.sampled.*; 

public class ClipDemo {
  public static void main(String[] args) {
     System.out.println();
     
     if(args.length != 1) {
        System.out.println(" Falsche Anzahl Argumente in Befehlszeile");
        System.out.println(" Aufruf: java ClipDemo <Klangdatei>");
        System.exit(0); 
     }


     try {
       // Input-Stream ffnen
       File datei = new File(args[0]);
       AudioInputStream eingabeStrom = AudioSystem.getAudioInputStream(datei); 

       // Informationen ber das Format ausgeben
       AudioFormat format = eingabeStrom.getFormat();

       System.out.println(" Datei       : " + datei.getName()); 
       System.out.println(" Kanaele     : " + format.getChannels());
       System.out.println(" Encoding    : " + format.getEncoding()); 
       System.out.println(" Sample-Rate : " + format.getSampleRate()); 

       // nur PCM wird von Line untersttzt; fr ULAW oder ALAW muss 
       // konvertiert werden
       if(format.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) {
         AudioFormat formatNeu = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 
         format.getSampleRate(), 2 * format.getSampleSizeInBits(),
         format.getChannels(),
         2 * format.getFrameSize(),
         format.getFrameRate(), true); 

         System.out.println(" Konvertiere nach PCM_SIGNED..."); 	  
         eingabeStrom = AudioSystem.getAudioInputStream(formatNeu,eingabeStrom); 
         format = formatNeu; 
       } 
       

       // eine Line vom Typ Clip anfordern und mit dem Eingabestrom
       // verbinden
       DataLine.Info clipInfo = new DataLine.Info(Clip.class, format);
       Clip clip = (Clip) AudioSystem.getLine(clipInfo);
       clip.open(eingabeStrom); 

       // abspielen
       clip.start();

       // regelmig prfen, ob der Clip zu Ende ist
       while(true) {
         try {
            Thread.sleep(500);
         } catch(Exception e) {
            System.err.println(e.getMessage());
         }
         
         if(clip.isRunning() == false)
            break; 
       }

       // schlieen
       clip.stop();
       clip.close(); 
     } catch(Exception e) {
       System.err.println(e.getMessage());
     }
  
    System.exit(0); 
  }
}
