/*
 * StreamingDemo.java - Abspielen einer Sound-Datei als Stream (beliebige Gre) 
 *
 */

import java.io.*;           
import javax.sound.sampled.*; 

public class StreamingDemo {
  public static void main(String[] args) {
     System.out.println();
     
     if(args.length != 1) {
        System.out.println(" Falsche Anzahl Argumente in Befehlszeile");
        System.out.println(" Aufruf: java StreamingDemo <Klangdatei>");
        System.exit(0); 
     }


     AudioInputStream quelle = null;
     AudioFormat format = null; 

     try {
       // Input-Stream ffnen
       File datei = new File(args[0]);
       quelle = AudioSystem.getAudioInputStream(datei); 

       // Informationen ber das Format ausgeben
       format = quelle.getFormat();

       System.out.println(" Datei       : " + datei.getName()); 
       System.out.println(" Kanaele     : " + format.getChannels());
       System.out.println(" Encoding    : " + format.getEncoding()); 
       System.out.println(" Sample-Rate : " + format.getSampleRate()); 

       // nur PCM wird von Line untersttzt; fr ULAW oder ALAW muss 
       // konvertiert werden
       if(format.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) {
          AudioFormat formatNeu = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 
                                                  format.getSampleRate(), 2 * format.getSampleSizeInBits(),
          format.getChannels(),
          2 * format.getFrameSize(),
          format.getFrameRate(), true); 

          System.out.println(" Konvertiere nach PCM_SIGNED..."); 	  
          quelle = AudioSystem.getAudioInputStream(formatNeu,quelle); 
          format = formatNeu; 
       } 
     } catch(Exception e) {
         System.err.println(e.getMessage()); 
     }
       
      // eine Line vom Typ SourceDataLine anfordern und ffnen
      SourceDataLine line = null;  
      DataLine.Info lineInfo = new DataLine.Info(SourceDataLine.class, format);

      try {
        line = (SourceDataLine) AudioSystem.getLine(lineInfo);
        line.open(format); 
        line.start(); 

        // in einer Schleife aus dem Eingabestream lesen und in die Line
        // schreiben
        int num = 0;
        byte[] audioPuffer = new byte[50000];

        while(num != -1) {
          try {
              num = quelle.read(audioPuffer, 0, audioPuffer.length);

              if(num >= 0) {
                 line.write(audioPuffer, 0, num); 
              }
          } catch(Exception e) {
              System.err.println(e.getMessage()); 
          }
        }

        // warten, bis die Line leer ist (= alles gespielt) 
        line.drain();

        // Line schlieen
        line.close();  
      } catch(Exception e) {
        System.err.println(e.getMessage()); 
      }

    System.exit(0);  // wegen JDK-Bug
  }
}
