/*
 * ZugriffDerby.java - Beispiel zum Anlegen und Fllen einer Tabelle mittels Java-DB (Derby) DBMS
 *
 */

import java.sql.*;
import java.util.*;

public class DemoDerby {
   public static void main(String[] args) {
      try {    // Treiber laden
          Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
      } catch(Exception e) {
          System.err.println(" Kann Treiber nicht laden: " + e);
      }

      // Verbindungseigenschaften
      Properties props = new Properties();
      props.put("user", "javauser");
      props.put("password", "lomu");
       
      Connection verbindung = null;
      String db = "jdbc:derby:musik";
      String user ="javauser";
      String pw = "lomu";

      try {
         // Verbindung herstellen und Java-DB starten
         // falls Umgebungsvariable derby.system.home nicht gesetzt ist, 
         // wird im aktuellen Verzeichnis die Datenbank angelegt
         verbindung = DriverManager.getConnection(db, props);
      } catch(SQLException e) {
         // Datenbank noch nicht vorhanden -> neu anlegen und einige Eintrge machen
         System.err.println(" Datenbank nicht vorhanden. Erzeugen ...");
      
         try {
            verbindung = DriverManager.getConnection(db + ";create=true", props);
         
            // Tabelle anlegen
            Statement stm = verbindung.createStatement();
            String sql = "create table interpreten(nummer int not null, " 
                     + "name varchar(30) not null, vorname varchar(30) not null, "
                     + "geburtsdatum date not null, geschlecht char not null, " 
                     + "plattenlabel varchar(30), umsatz decimal(9,2), " 
                     + "primary key(nummer))";

            stm.execute(sql);
            System.out.println(" Tabelle 'interpreten' in Datenbank 'musik' erzeugt");
          
            // einige Eintrge erzeugen
            sql ="insert into interpreten values(1, 'Meier', 'Kurt', '1981-05-17', 'm', 'Arion', 0)";
            stm.execute(sql);
         
            sql ="insert into interpreten values(2, 'Navier', 'Sara', '1986-03-15', 'w', 'DRT', 378000.50)";
            stm.execute(sql);
            System.out.println(" Eintraege in Tabelle 'interpreten' erzeugt\n");

         } catch(Exception ex) {
            System.out.println(e);
            System.exit(1);
         }
      }
         
      try {
         // Query ausfhren
         Statement stm = verbindung.createStatement();
         String sql ="SELECT * from interpreten";
         ResultSet ergebnis = stm.executeQuery(sql);

         // Ergebnis drucken
         String name,vorname,label,geschlecht;
         java.sql.Date geburt; //java.sql.Date
         int num;
         double umsatz;

         System.out.println(" Inhalt von Tabelle interpreten:");

         while(ergebnis.next() == true) {
            num = ergebnis.getInt("nummer");
            name = ergebnis.getString("name");
            vorname = ergebnis.getString("vorname");
            geburt = ergebnis.getDate("geburtsdatum");
            umsatz = ergebnis.getDouble("umsatz");
            label = ergebnis.getString("plattenlabel");
            geschlecht = ergebnis.getString("geschlecht"); 

            System.out.println(num + "\t" + name + "\t" + 
                         vorname + "\t" + geburt  + "\t" +
                         geschlecht + "\t" + label + "\t" + umsatz);
         }
		 
         verbindung.close();
         	
         
         // derby herunterfahren
         try {	
            DriverManager.getConnection("jdbc:derby:;shutdown=true");
         } catch (SQLException e) {   
            // es wird immer eine Exception hier von Derby geworfen; das ist normal;		  
         }
         
      } catch(Exception e) {
         System.err.println(" " + e.getMessage());
      }
   }
}
