/*
 * DemoMySQL.java - Beispiel zum Einfgen eines Eintrags und Auslesen aller Daten aus der Tabelle interpreten  mit MySQL
 *
 */
import java.awt.BorderLayout;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;

public class DemoMySQL extends JFrame {
    
   public DemoMySQL() {
       super();
       setTitle("Musikerdatenbank");
       setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
       
       JTable tabelle = erzeugeJTable();
       this.getContentPane().add(new JScrollPane(tabelle), BorderLayout.CENTER);
       
   } 
   
  /** Daten aus SQL Tabelle lesen, einen Dummy-Eintrag hinzufgen und daraus
    * ein JTable erzeugen 
    */ 
  private JTable erzeugeJTable() {
      try {                                              // Treiber laden
         Class.forName("com.mysql.jdbc.Driver");
      } catch(Exception e) {
         System.err.println(" Kann Treiber nicht laden: " + e);
      }

      Connection verbindung = null;
      String db = "jdbc:mysql://localhost/musik";
      String user ="javauser";
      String pw = "lomu";

      try  {
         // Verbindung herstellen
         verbindung = DriverManager.getConnection(db,user,pw);

	      // einen weiteren Eintrag mit heutigem Datum als Geburtsdatum hinzufgen
	      Statement stm = verbindung.createStatement();
         LocalDate aktDatum = LocalDate.now();
         String datum        = aktDatum.format(DateTimeFormatter.ISO_LOCAL_DATE);
	      String cmd = "INSERT into interpreten values(null,'Testfuzzy', 'Erwin','" + datum + "','m', 'FuzzyRecords', 0.0)";
	      stm.execute(cmd);    
		 
         // Query ausfhren
         stm = verbindung.createStatement();
         String sql ="SELECT * from interpreten";
         ResultSet ergebnis = stm.executeQuery(sql);

         List<Eintrag> tabellenEintraege = new ArrayList<Eintrag>();
         

         while(ergebnis.next()) {
            Eintrag eintrag      = new Eintrag();
            eintrag.nummer       = ergebnis.getInt("nummer"); 
            eintrag.name         = ergebnis.getString("name");
            eintrag.vorname      = ergebnis.getString("vorname");
            eintrag.geburtsDatum = ergebnis.getDate("geburtsdatum");
            eintrag.geschlecht   = ergebnis.getString("geschlecht");
            eintrag.umsatz       = ergebnis.getDouble("umsatz");
            eintrag.label        = ergebnis.getString("plattenlabel");
           
            tabellenEintraege.add(eintrag);             
         }
       
         verbindung.close();
         
         // JTable erzeugen
         String[] spaltenNamen = {"Nummer", "Name", "Vorname", "Geburtsdatum", "Geschlecht", 
                                  "Plattenlabel", "Umsatz"};
         Object[][] daten = new Object[tabellenEintraege.size()][spaltenNamen.length];
         
         for(int i = 0; i < tabellenEintraege.size(); i++) {
            Eintrag eintrag = tabellenEintraege.get(i);
            daten[i][0] = eintrag.nummer;
            daten[i][1] = eintrag.name;
            daten[i][2] = eintrag.vorname;
            daten[i][3] = eintrag.geburtsDatum;
            daten[i][4] = eintrag.geschlecht;
            daten[i][5] = eintrag.label;
            daten[i][6] = eintrag.umsatz;
         }
         
         return new JTable(daten, spaltenNamen);
         
      } catch(Exception e) {
         System.err.println(" " + e.getMessage());
         return null; 
      }
   }
   
   
   public static void main(String args[]) {
      DemoMySQL hauptfenster = new DemoMySQL();
      hauptfenster.setSize(600, 300);                     
      hauptfenster.setLocation(200,300);   
      hauptfenster.setVisible(true); 
   }
   
   
   private class Eintrag {
       int nummer;
       String name;
       String vorname;
       Date geburtsDatum;
       String geschlecht;
       String label;
       double umsatz; 
   }
   
}
