/*
 * FraktalDemoRunnable.java - Fraktalberechnung mit Thread-Einsatz via Runnable-Schnittstelle
 *
 */

import java.awt.*;
import java.awt.geom.*; 
import java.awt.event.*;
import javax.swing.*;
import java.util.*; 

public class FraktalDemoRunnable  {
   public static void main(String[] args) {
      FraktalFenster fenster = new FraktalFenster();
      fenster.validate();

      // Fenster zentrieren
      Dimension bildschirmGroee = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension fensterGroee = fenster.getSize();

      if (fensterGroee.height > bildschirmGroee.height) 
      fensterGroee.height = bildschirmGroee.height;

      if (fensterGroee.width > bildschirmGroee.width) 
      fensterGroee.width = bildschirmGroee.width;

      fenster.setLocation((bildschirmGroee.width - fensterGroee.width) / 2, 
                          (bildschirmGroee.height - fensterGroee.height) / 2);
      fenster.setVisible(true);
   }
}

class FraktalFenster extends JFrame implements ActionListener, Runnable {
   private JPanel contentPane;
   private JPanel canvas;
   private Thread fraktal;
   private BorderLayout borderLayout = new BorderLayout();
   private ButtonGroup auswahl; 
   private boolean weiter; 

   // Konstruktor
   public FraktalFenster() {
      contentPane = (JPanel) this.getContentPane();
      contentPane.setLayout(borderLayout);
      setSize(new Dimension(400, 400));
      setResizable(false);
      setTitle("Fraktal-Demo");

      JPanel tmp = new JPanel();
      JButton start = new JButton("Start");
      JButton stop  = new JButton("Stopp");
      start.addActionListener(this);
      stop.addActionListener(this); 
      tmp.add(start);
      tmp.add(stop);
      contentPane.add(tmp,BorderLayout.SOUTH); 

      JRadioButton juliaMenge = new JRadioButton("Julia-Menge", true);
      juliaMenge.setActionCommand("juliaMenge"); 
      JRadioButton kingsDream = new JRadioButton("King's Dream", false);
      kingsDream.setActionCommand("kingsDream"); 
      JRadioButton lorenz = new JRadioButton("Lorenz farbig", false);
      lorenz.setActionCommand("lorenzFarbig"); 

      JPanel auswahlPanel = new JPanel();
      auswahlPanel.add(juliaMenge);
      auswahlPanel.add(kingsDream);
      auswahlPanel.add(lorenz);
      contentPane.add(auswahlPanel, BorderLayout.NORTH); 

      auswahl = new ButtonGroup();
      auswahl.add(juliaMenge);
      auswahl.add(kingsDream);
      auswahl.add(lorenz);

      canvas = new JPanel();
      canvas.setSize(400,300); 
      canvas.setBackground(Color.BLACK); 
      contentPane.add(canvas, BorderLayout.CENTER); 
      
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);      
   }

   private void lorenzFarbig() {
      double a = 10.0;
      double b = 8 / 3;
      double c = 28.0;
      double d = 0.003;

      double x_neu,x_alt, y_neu, y_alt, z_neu, z_alt;
      int pos_x, pos_y;

      x_alt = 0.1;
      y_alt = 0.1;
      z_alt = 0.5;

      Graphics g = canvas.getGraphics();    
      g.setColor(Color.BLACK);
      g.fillRect(0,0,400,400);
      g.setColor(Color.RED); 

      int num = 0;

      Color[] farben = new Color[5];

      farben[0] = Color.RED;
      farben[1] = Color.GREEN;
      farben[2] = Color.PINK;
      farben[3] = Color.YELLOW;
      farben[4] = Color.BLUE; 
   
      Random wrfel = new Random();

      while(weiter)  {
         x_neu = x_alt + d * a * (y_alt - x_alt);    
         y_neu = y_alt + d * (x_alt * (c - z_alt) - y_alt);
         z_neu = z_alt + d * (x_alt * y_alt - b * z_alt);

         pos_x = (int) (200 + x_neu * 100);
         pos_y = (int) (250 + y_neu * 100);

         if(pos_x >= 0 && pos_x <= 400 && pos_y >= 0 && pos_y <= 400) { 
            g.fillRect(pos_x, pos_y, 1 , 1 ); 
            num++;
             
            if(num > 50000) {
               int farbe = wrfel.nextInt(5); 
               g.setColor(farben[farbe]); 
               num = 0; 
            }
         }

         x_alt = x_neu;
         y_alt = y_neu;
         z_alt = z_neu;
      }
   }

   private void kingsDream()  {
      Graphics g = canvas.getGraphics();	
      g.setColor(Color.BLACK);
      g.fillRect(0,0,400,300);

      g.setColor(Color.RED); 

      double a = -0.966918;
      double b = 2.879879;
      double c = 0.765145;
      double d = 0.744728;

      double x_alt = 0.12; 
      double y_alt = 0.5;

      double x_neu;
      double y_neu;
      int pos_x, pos_y; 
      weiter = true;

      while(weiter)  {
         x_neu = Math.sin(y_alt * b) + c * Math.sin(x_alt * b);
         y_neu = Math.sin(x_alt * a) + d * Math.sin(y_alt * a);

         x_alt = x_neu;
         y_alt = y_neu; 

         pos_x = (int) (200 + x_neu * 100);
         pos_y = (int) (150 + y_neu * 100);

         if(pos_x >= 0 && pos_x <= 400 && pos_y >= 0 && pos_y <= 300)
            g.fillRect(pos_x, pos_y, 1 , 1 ); 
      }
   }

   private void juliaMenge()  {
      Graphics g = canvas.getGraphics();	
      g.setColor(Color.BLACK);
      g.fillRect(0,0,400,300);
      g.setColor(Color.RED); 

      double a = -0.012;
      double b = 0.74;
      double x,y; 
      double x_2,y_2; 

      Dimension gre = this.getSize();

      for(int i = 0; i < gre.width; i++)
         for(int j = 0; j < gre.height; j++) {
            x = 0.0001 * i;
            y = 0.0001 * j;

            x_2 = x * x;
            y_2 = y * y; 

            for(int n = 0; n < 100; n++) {
               if((x_2 + y_2)  > 10000)
                  break;
  
               y = 2 * x * y + b;
               x = x_2 - y_2 + a;

               x_2 = x * x;
               y_2 = y * y; 
            }

            if(Math.abs(x) < 1)
               g.setColor(Color.RED);
            else
               g.setColor(Color.BLACK);
  
            g.fillRect(i,j,1,1);  
         }
      }

   public void run()  {
      ButtonModel bm = auswahl.getSelection(); 
  
      if(bm.getActionCommand().equals("lorenzFarbig"))
         lorenzFarbig(); 

      if(bm.getActionCommand().equals("juliaMenge"))
         juliaMenge(); 

      if(bm.getActionCommand().equals("kingsDream"))
         kingsDream(); 
   }

   public void actionPerformed(ActionEvent e)  {
      String cmd = e.getActionCommand();

      if((cmd.equals("Start") == true) && (fraktal == null))  {
         fraktal = new Thread(this);
         fraktal.start(); 
         weiter = true;
      }

      if((cmd.equals("Stopp") == true) && (fraktal != null)) {
         weiter = false;
         fraktal = null;
      }
   }
}


