/*
 * SwingWorkerDemo.java - Einsatz von SwingWorker
 *
 */

import java.awt.*;
import java.awt.geom.*; 
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

public class SwingWorkerDemo  {
  public static void main(String[] args) {
    DemoFenster fenster = new DemoFenster();
    fenster.validate();
    fenster.setVisible(true);
  }
}
 
class DemoFenster extends JFrame implements ActionListener {
   private JPanel contentPane;
   private JTextArea tfDateiliste;
   private JButton btnStartStopp;
   private TIFSuche tifSuche; 
   private JTextField tfStartverzeichnis;
   
   // Konstruktor
   public DemoFenster() {
      contentPane = (JPanel) this.getContentPane();
      contentPane.setLayout(new BorderLayout());
      setSize(new Dimension(500, 200));
      setTitle("SwingWorker-Demo");
      
      JPanel tmp = new JPanel();
      tfStartverzeichnis = new JTextField("c:\\temp");
      tfStartverzeichnis.setColumns(30);
      tmp.add(new JLabel("Startverzeichnis:"));
      tmp.add(tfStartverzeichnis); 
      contentPane.add(tmp, BorderLayout.NORTH);
      
      tfDateiliste = new JTextArea();
      JScrollPane scrollPane = new JScrollPane(tfDateiliste);
      contentPane.add(scrollPane, BorderLayout.CENTER);
      tmp = new JPanel();

      btnStartStopp = new JButton("TIF suchen");
      btnStartStopp.addActionListener(this);
      tmp.add(btnStartStopp);
      contentPane.add(tmp,BorderLayout.SOUTH); 

      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }

   // Suche starten/stoppen
   public void actionPerformed(ActionEvent e)  {
      String cmd = e.getActionCommand();

      if((cmd.equals("TIF suchen") == true))  {
         btnStartStopp.setText("Suche beenden");
         tifSuche = new TIFSuche(tfStartverzeichnis.getText(), tfDateiliste, btnStartStopp);
         tifSuche.execute();
      }
      else {
         tifSuche.cancel(true); 
         btnStartStopp.setText("TIF suchen");
      }

   }
}

// SwingWorker Klasse mit Hintergrund-Aktivitt
class TIFSuche extends SwingWorker<Integer, String> {
   private JTextArea tfDateiliste;
   private int zaehler = 0; 
   private JButton btnStartStopp;
   private String pfad; 
   
   TIFSuche(String pfad, JTextArea liste, JButton btnStartStopp) {
      super();
      tfDateiliste = liste;
      this.btnStartStopp = btnStartStopp;
      this.pfad = pfad; 
   }
   
   
   // Partition C durchsuchen nach TIF-Dateien und  anzeigen
   public Integer doInBackground() {
         
     try {
        File f = new File(pfad);
        sucheTIFDateien(f);
     }
     catch(Exception e) {
        System.err.println(e.getMessage()); 
     }
     
     return new Integer(zaehler); 
   }
   
   // wird von Event-Thread aufgerufen
   protected void process(java.util.List<String> dateiNamen) {
      System.out.println(" process() aufgerufen ");
      for(String s : dateiNamen)
         tfDateiliste.append(s + "\n");
   }
   
   
   // Ergebnis von doInBackground() ausgeben; 
   // wird vom Event-Thread aufgerufen
   protected void done() {
      try {
         tfDateiliste.append(" Anzahl Treffer: " + get() + "\n");
         btnStartStopp.setText("TIF suchen");
      } catch(java.util.concurrent.CancellationException e) {
      } catch(Exception e) {
         System.err.println(e.getMessage()); 
      }
   }
   
   
   // sucht rekursiv nach TIF-Dateien
   private void sucheTIFDateien(File f) throws IOException {
      if(isCancelled())
         return;
     
      if(f.isDirectory()) {
         File[] files = f.listFiles();
       
         for(int i = 0; i < files.length; i++) 
            sucheTIFDateien(files[i]);

      } else {
         if(f.getName().toLowerCase().endsWith(".tif")) {
           System.out.println(" gefunden: " + f.getName()); 
           publish(f.getCanonicalPath()); 
           zaehler++;
         }
      }
   }
}
